use crate::library::Video;
use based::ui::components::AppBar;
use based::ui::prelude::*;
use based::{
    auth::User,
    format::{format_date, format_number, format_seconds_to_hhmmss},
    request::{RequestContext, StringResponse},
};
use maud::{html, PreEscaped, Render};

pub async fn render_page(
    ctx: RequestContext,
    content: PreEscaped<String>,
    title: &str,
    user: Option<User>,
) -> StringResponse {
    based::ui::render_page(
        content,
        title,
        ctx,
        &based::ui::components::Shell::new(
            html! {
                script src="https://cdn.tailwindcss.com" {};
                script src="/assets/htmx.min.js" {};
                meta name="viewport" content="width=device-width, initial-scale=1.0";
            },
            html! {
                (script(include_str!("../scripts/header.js")));
                    (AppBar("WatchDogs", user))
            },
            Some(String::from("bg-black text-white")),
        ),
    )
    .await
}

pub async fn video_element_wide(video: &Video) -> PreEscaped<String> {
    html!(
        a href=(format!("/watch?v={}", video.id)) class="flex items-center w-full my-4 bg-gray-900 hover:bg-gray-800 shadow-lg rounded-lg overflow-hidden" {
            div class="flex-shrink-0 relative" {
                img width="480" src=(format!("/video/thumbnail?v={}", video.id)) class="aspect-video w-32 w-48 rounded-md object-cover";
                span class="absolute bottom-2 right-2 bg-black text-white text-xs px-2 py-1 rounded-sm opacity-90" {
                    (( format_seconds_to_hhmmss(video.duration) ))
                };
            };

            div class="flex flex-col flex-grow ml-2" {
                (Margin(Text(&video.title).large().semibold().max_lines(LineClamp::_3).title(&video.title).align(TextAlignment::Start)
            ).bottom(ScreenValue::_1)
        )

                @if let Some(meta) = video.youtube_meta().await {
                div class="text-sm text-gray-400 mb-2" {
                    (Span(&meta.uploader_name).medium())
                    (Span(" - "))
                    (Span(&format_date(&meta.upload_date)))
                };

                div class="text-sm text-gray-400" {
                    (Span(&format_number(meta.views)))
                    (Span(" views"))
                };
            };
        };
        };
    )
}

pub fn video_thumbnail_with_time(video: &Video) -> PreEscaped<String> {
    html! {
            div class="relative" {
            img width="480" src=(format!("/video/thumbnail?v={}", video.id)) class="w-full h-auto object-cover aspect-video";
            span class="absolute bottom-2 right-2 bg-black text-white text-xs px-2 py-1 rounded-sm opacity-90" {
                (( format_seconds_to_hhmmss(video.duration) ))
            };
        };
    }
}

pub async fn video_element(video: &mut Video) -> PreEscaped<String> {
    Margin(Context(MaxWidth(
        ScreenValue::_90,
        MaxHeight(
            ScreenValue::_60,
            Aspect::video(Link(
                &format!("/watch?v={}", video.id),
                Context(
                    Hover(Background(Gray::_800, Nothing())).on(Background(
                        Gray::_900,
                        Shadow::large(
                            Rounded(
                                Div().push(video_thumbnail_with_time(&video)).push(Context(
                                    Rounded(Shadow::large(
                                        Padding(
                                            Text(&video.title)
                                                .base_size()
                                                .semibold()
                                                .align(TextAlignment::Start)
                                                .title(&video.title)
                                                .max_lines(LineClamp::_1)
                                                .overflow(TextOverflow::Truncate),
                                        )
                                        .y(ScreenValue::_4)
                                        .x(ScreenValue::_2),
                                    ))
                                    .size(Size::Large),
                                )),
                            )
                            .size(Size::Large),
                        ),
                    )),
                ),
            )),
        ),
    )))
    .y(ScreenValue::_6)
    .render()
}
