use based::ui::htmx::HTMXAttributes;
use based::ui::prelude::*;
use based::{
    auth::MaybeUser,
    request::{
        api::{vec_to_api, FallibleApiResponse},
        RequestContext, StringResponse,
    },
};
use maud::html;
use rocket::{get, State};
use serde_json::json;

use crate::{library::Library, pages::components::video_element};

use super::{
    api_response,
    components::{render_page, video_element_wide},
};

#[get("/search?<query>&<offset>")]
pub async fn search(
    query: &str,
    offset: Option<i64>,
    library: &State<Library>,
) -> Option<serde_json::Value> {
    const NUM_OF_RESULTS: i64 = 20;

    // get start parameter for search result chunks
    let start = offset.unwrap_or(0);

    let mut video_matches = library.search_video(query, start, NUM_OF_RESULTS).await;

    Some(json!(vec_to_api(&mut video_matches).await))
}

#[get("/latest.json")]
pub async fn latest_api(library: &State<Library>) -> FallibleApiResponse {
    let videos = library.get_newly_added(20).await;
    let vid_api = vec_to_api(&videos).await;
    return Ok(serde_json::json!(vid_api));
}

#[get("/latest")]
pub async fn latest_page(
    ctx: RequestContext,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    let videos = library.get_newly_added(20).await;

    let content = html!(
        h1 class="text-center text-4xl font-extrabold leading-tight mt-6 mb-2" { "Recent videos" };
        div class="p-6" {
        @for mut vid in videos {
            ( video_element_wide(&mut vid).await );
        };
    };
    );

    render_page(ctx, content, "Recent videos", user.into()).await
}

#[get("/d/<dir>")]
pub async fn channel_page(
    ctx: RequestContext,
    dir: &str,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    if dir.ends_with(".json") {
        let dir_videos = library
            .get_directory_videos(dir.split_once(".json").map(|x| x.0).unwrap_or_default())
            .await;
        return api_response(&json!(vec_to_api(&dir_videos).await));
    }

    let dir_videos = library.get_directory_videos(dir).await;

    let content = html!(
        h1 class="text-center text-4xl font-extrabold leading-tight mt-4 mb-2" { (dir) };
        div class="p-6" {
        @for mut vid in dir_videos {
            ( video_element_wide(&mut vid).await );
        };
    };
    );

    render_page(ctx, content, dir, user.into()).await
}

#[get("/")]
pub async fn index_page(
    ctx: RequestContext,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    let content = html!(
        h1 class="text-center text-4xl font-extrabold leading-tight mt-8" { "Random Videos" };
        div class="lg:grid grid-cols-3 gap-6 p-6 mb-4" {
        @for mut vid in library.get_random_videos(3).await {
            ( video_element(&mut vid).await );
        };
        };

        h1 class="text-center text-4xl font-extrabold leading-tight mt-16" { a href="/latest" { "Latest Videos" };};
        div class="lg:grid grid-cols-3 gap-6 p-6 mb-4" {
        @for mut vid in library.get_newly_added(3).await {
            ( video_element(&mut vid).await );
        };
        };

        h1 class="text-center text-4xl font-extrabold leading-tight mt-8" { "Directories:" };
        div class="flex flex-wrap p-10" {
        @for dir in library.get_directories().await {


            (Margin(
                Padding(
                    Hover(Background(Purple::_600, Nothing())).on(
                    Background(Purple::_500,
                        Rounded(
                            // TODO : Implement cursor-pointer
                            Link(
                                &format!("/d/{dir}"),
                                Text(&dir).white()
                            ).use_htmx()
                        ).size(Size::Full)
                    ))
            ).x(ScreenValue::_3).y(ScreenValue::_2)).all(ScreenValue::_2))
            br;
        };
    };
    );

    render_page(ctx, content, "WatchDogs", user.into()).await
}
